;;; Copyright (c) 2000 Matteo Frigo
;;; Copyright (c) 2000 Biomedin, s.r.l.
;;;
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2 of the License, or
;;; (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;;;

;; $Id: gmeteor-lib.scm,v 1.1 2001/04/27 00:24:59 matley Exp $

;; This file contains miscellaneous utility functions

(define-module (gmeteor-lib)
  :use-module (gmeteor))

(export frequency-response first-order-interpolator)

;;------------------------------------------------------------
;; compute the frequency response from the coefficients
;;------------------------------------------------------------
(define (frequency-response coefficients)
  (let* ((l (vector-length coefficients))
	 (l-1 (- l 1))
	 (pi (* 4 (atan 1)))
	 (2pi (* 2 pi)))
    (lambda (omega)
      (* (exp (* -i pi l-1 omega))
	 (sigma l
		(lambda (j)
		  (* (vector-ref coefficients j)
		     (exp (* 2pi +i j omega)))))))))

;;------------------------------------------------------------
;; interpolate a function from tabulated values.
;;------------------------------------------------------------

;; Example: (first-order-interpolator '((0 1) (1 2) (4 3))
;; produces a function f such that f(0)=1, f(1)=2, f(4)=3, and
;; the remaining values in the range [0..4] are interpolated.

(define (first-order-interpolator spec-list)
  (define (make-interpolator lx ly hx hy cont)
    (let ((k (/ (- hy ly) (- hx lx))))
      (lambda (x)
	(if (<= x hx)
	    (+ ly (* (- x lx) k))
	    (cont x)))))
  (define spec/x car)
  (define spec/y cadr)

  (if (null? (cdr spec-list))
      (lambda (x) (error "interpolator"))
      (make-interpolator (spec/x (car spec-list)) (spec/y (car spec-list))
			 (spec/x (cadr spec-list)) (spec/y (cadr spec-list))
			 (first-order-interpolator (cdr spec-list)))))


