;;; Copyright (c) 2000 Matteo Frigo
;;; Copyright (c) 2000 Biomedin, s.r.l.
;;;
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2 of the License, or
;;; (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;;;

;; $Id: gmeteor-simple.scm,v 1.4 2003/09/28 00:00:24 matley Exp $

(define-module (gmeteor-simple)
  :use-module (gmeteor)
  :use-module (gmeteor-core))

(export 
 *the-filter* 
 filter-length sampling-frequency
 limit-< limit-> limit-= limit-<= limit->=
 limit-derivative-< limit-derivative-> limit-derivative-= 
 limit-derivative-<= limit->=
 concave-up concave-down upward downward
 output-file plot-file 
 sine-symmetry cosine-symmetry
 plot-absolute 
 verbose go 
 grid-density
 plot-grid-density
 title harmonic
 iteration-limit
 solve *coefficients* *y* *H* *result*)

 
;;------------------------------------------------------------
;; the simple user interface
;;------------------------------------------------------------
(define *the-filter* (make-filter))

;; we store the result of filter design into these global variables
(define *coefficients* 'not-yet)
(define *y* 'not-yet)
(define *H* 'not-yet)
(define *result* 'not-yet)

(define filter-length (*the-filter* 'set-filter-length!*))
(define sampling-frequency (*the-filter* 'set-sampling-frequency!*))

(define (limit-generic sense)
  (lambda (i bound . weight)
    (let ((x (if (null? weight) 1 (car weight))))
      ((*the-filter* 'add-specification!) 'limit i sense bound x))))
(define limit-< (limit-generic '<))
(define limit-<= limit-<)
(define limit-> (limit-generic '>))
(define limit->= limit->)
(define limit-= (limit-generic '=))

(define (limit-<> i bound> bound< . weight)
  (let ((x (if (null? weight) 1 (car weight))))
    ((*the-filter* 'add-specification!) 'limit i '< bound< x)
    ((*the-filter* 'add-specification!) 'limit i '> bound> x)))

(define (limit-derivative-generic sense)
  (lambda (order i bound . weight)
    (let ((x (if (null? weight) 1 (car weight))))
      ((*the-filter* 'add-specification!) 'derivative i sense order bound x))))
(define limit-derivative-< (limit-derivative-generic '<))
(define limit-derivative-<= limit-derivative-<)
(define limit-derivative-> (limit-derivative-generic '>))
(define limit-derivative->= limit-derivative->)
(define limit-derivative-= (limit-derivative-generic '=))

(define (concave-up i) (limit-derivative-> 2 i 0 0))
(define (concave-down i) (limit-derivative-< 2 i 0 0))
(define (upward i) (limit-derivative-> 1 i 0 0))
(define (downward i) (limit-derivative-< 1 i 0 0))

(define output-file (*the-filter* 'set-output-file!*))

(define plot-file (*the-filter* 'set-plot-file!*))

(define (sine-symmetry) ((*the-filter* 'set-symmetry!) 'sine))
(define (cosine-symmetry) ((*the-filter* 'set-symmetry!) 'cosine))

(define plot-absolute (*the-filter* 'set-plot-absolute!*))

(define verbose (*the-filter* 'set-verbose!))

(define grid-density (*the-filter* 'set-grid-density!*))

(define plot-grid-density (*the-filter* 'set-plot-grid-density!*))

(define iteration-limit (*the-filter* 'set-iteration-limit!*))

(define title (*the-filter* 'set-title!))

(define harmonic (*the-filter* 'set-harmonic!))

(define (solve)
  (begin
    ((*the-filter* 'solve!))
    (set! *coefficients* ((*the-filter* 'get-coefficients)))
    (set! *result* ((*the-filter* 'get-result)))
    (set! *H* ((*the-filter* 'get-H)))
    (set! *y* ((*the-filter* 'get-y)))))
(define (go)
  (begin
    (solve)
    ((*the-filter* 'display))))
    
