/*
 * Copyright (c) 2000 Matteo Frigo
 * Copyright (c) 2000 Biomedin s. r. l.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/* $Id: gmeteor.c,v 1.2 2001/05/08 18:20:15 matley Exp $ */
#include "config.h"

#ifndef HAVE_GUILE_GH_H
#error "cannot find <guile/gh.h>.  Guile is not installed properly."
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <guile/gh.h>
#include "simplex.h"

#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif

/*
 * two-arguments gh_scm2doubles as in guile-1.3.4.  In older versions
 * of guile, gh_scm2doubles takes only one argument.  We will
 * replace this routine once guile stabilizes.
 */
static void gmeteor_scm2doubles(SCM vector, double *result)
{
     int i, n;
     n = gh_vector_length(vector);
     for (i = 0; i < n; ++i)
	  result[i] = gh_scm2double(
	       gh_vector_ref(vector, gh_int2scm(i)));
}

static void gmeteor_doubles2scm(SCM dst, double *src, int n)
{
     int i;
     for (i = 0; i < n; ++i)
	  gh_vector_set_x(dst, gh_int2scm(i), gh_double2scm(src[i]));
}


/* this function makes the simplex solver accessible from guile */
static SCM s_simplex_solve(SCM size,
			   SCM objective_function, 
			   SCM equation_function,
			   SCM obj,
			   SCM primduals)
{
     int nrow = gh_scm2int(gh_car(size));
     int ncol = gh_scm2int(gh_cadr(size));
     int il = gh_scm2int(gh_cddr(size));
     double *row;
     double rhs;
     SIMPLEX *s;
     SCM p = SCM_BOOL_F;
     SCM primals = gh_car(primduals);
     SCM duals = gh_cdr(primduals);
     int res;
     int i;
     SCM code = SCM_BOOL_F;
     s = simplex_make(nrow, ncol);
     row = malloc(ncol * sizeof(double));
     simplex_set_iteration_limit(s, il);

     gmeteor_scm2doubles(objective_function, row);
     simplex_set_objective(s, row);

     for (i = 0; i < nrow; ++i) {
	  SCM ii = gh_int2scm(i);
	  SCM equation = gh_call1(equation_function, ii);
	  rhs = gh_scm2double(gh_cdr(equation));
	  gmeteor_scm2doubles(gh_car(equation), row);
	  simplex_add_equation(s, row, rhs);
     }
     res = simplex_solve(s);
     switch (res) {
	 case G_OPTIMAL:      
	      code = gh_symbol2scm("optimal");
	      gmeteor_doubles2scm(duals, s->duals, nrow);
	      gmeteor_doubles2scm(primals, s->primals, ncol);
	      gh_vector_set_x(obj, gh_int2scm(0), 
			      gh_double2scm(s->objective_value));
	      break;
	 case G_INFEASIBLE:      
	      code = gh_symbol2scm("infeasible");
	      break;
	 case G_UNBOUNDED:      
	      code = gh_symbol2scm("unbounded");
	      break;
	 case G_ITERATION_LIMIT:      
	      code = gh_symbol2scm("iteration-limit-reached");
	      break;
	 default:
	      code = gh_symbol2scm("unknown");
	      break;
     }

     simplex_free(s);
     free(row);
     return code;
}

void gmeteor_init_gmeteor(int argc, char *argv[])
{
     gh_new_procedure5_0("%simplex-solve", s_simplex_solve);
}
